package com.amap.app.cloud.activity;

import android.app.Activity;
import android.content.Context;
import android.graphics.drawable.ColorDrawable;
import android.view.GestureDetector;
import android.view.GestureDetector.OnGestureListener;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.Button;
import android.widget.PopupWindow;

import com.amap.app.cloud.R;
import com.amap.app.cloud.constants.CloudConstants;

public class BottomPopupWindow extends PopupWindow implements OnGestureListener {

	private Button btn1, btn2, btn_cancel;
	private View mMenuView;

	private GestureDetector gestureScanner;

	public BottomPopupWindow(Activity context, int optionType,
			OnClickListener itemsOnClick) {
		super(context);
		LayoutInflater inflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		mMenuView = inflater.inflate(R.layout.popupwindow_bottom, null);
		gestureScanner = new GestureDetector(context, this);
		btn1 = (Button) mMenuView.findViewById(R.id.popwin_btn1);
		btn2 = (Button) mMenuView.findViewById(R.id.popwin_btn2);
		btn_cancel = (Button) mMenuView.findViewById(R.id.popwin_btn_cancel);
		// 取消按钮
		btn_cancel.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// 销毁弹出框
				dismiss();
			}
		});

		if (optionType == CloudConstants.OPTION_POPWIN_PIC) {
			btn1.setText(context.getResources().getString(R.string.takePhoto));
			btn2.setText(context.getResources().getString(R.string.pickPhoto));
		}

		if (optionType == CloudConstants.OPTION_POPWIN_LOCATION) {
			btn1.setText(context.getResources().getString(R.string.mylocation));
			btn2.setText(context.getResources().getString(
					R.string.locationfrommap));
		}
		// 设置按钮监听
		btn1.setOnClickListener(itemsOnClick);
		btn2.setOnClickListener(itemsOnClick);
		// 设置SelectPicPopupWindow的View
		this.setContentView(mMenuView);
		// 设置SelectPicPopupWindow弹出窗体的宽
		this.setWidth(LayoutParams.MATCH_PARENT);
		// 设置SelectPicPopupWindow弹出窗体的高
		this.setHeight(LayoutParams.WRAP_CONTENT);
		// 设置SelectPicPopupWindow弹出窗体可点击
		this.setFocusable(true);
		// 设置SelectPicPopupWindow弹出窗体动画效果
		this.setAnimationStyle(R.style.AnimBottom);
		// 实例化一个ColorDrawable颜色为半透明
		ColorDrawable dw = new ColorDrawable(0xb0000000);
		// 设置SelectPicPopupWindow弹出窗体的背景
		this.setBackgroundDrawable(dw);
		// mMenuView添加OnTouchListener监听判断获取触屏位置如果在选择框外面则销毁弹出框
		mMenuView.setOnTouchListener(new OnTouchListener() {

			public boolean onTouch(View v, MotionEvent event) {

				gestureScanner.onTouchEvent(event);
				return true;
			}
		});

	}

	@Override
	public boolean onDown(MotionEvent e) {
		return false;
	}

	@Override
	public void onShowPress(MotionEvent e) {

	}

	@Override
	public boolean onSingleTapUp(MotionEvent e) {
		int height = mMenuView.findViewById(R.id.pop_layout).getTop();
		if (e.getY() < height) {
			dismiss();
		}
		return true;
	}

	@Override
	public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX,
			float distanceY) {
		return false;
	}

	@Override
	public void onLongPress(MotionEvent e) {

	}

	@Override
	public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX,
			float velocityY) {
		if (e2.getY() - e1.getY() > CloudConstants.SCROLL_MINPIX) {
			dismiss();
		} else {
			int height = mMenuView.findViewById(R.id.pop_layout).getTop();
			if (e1.getY() < height) {
				dismiss();
			}
		}
		return true;
	}

}
