package com.amap.app.cloud.activity;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.Button;
import android.widget.EditText;

import com.amap.app.cloud.R;
import com.amap.app.cloud.net.Protocal;
import com.amap.app.cloud.util.ConfigableConst;
import com.amap.app.cloud.util.Util;

public class LoginActivity extends Activity implements OnClickListener {

	private Button btLogin;
	private EditText mUserName;
	private EditText mPassword;
	private Protocal mProtocal;
	private String mStrName;
	private String mStrPassword;

	private static final int LOGGING = 1;
	private static final int LOGINFINISH = 0;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.activity_login);
		mUserName = (EditText) findViewById(R.id.etPoiName);
		mPassword = (EditText) findViewById(R.id.etPoiAddr);
		btLogin = (Button) findViewById(R.id.btLogin);

		mProtocal = ConfigableConst.getProtocalInstance(this
				.getApplicationContext());
		btLogin.setOnClickListener(this);
		SharedPreferences sharedata = getSharedPreferences("SP", MODE_PRIVATE);
		String mNameStr = sharedata.getString("NAME", null);
		String mPasswordStr = sharedata.getString("PASSWORD", null);
		mUserName.setText(mNameStr);
		mPassword.setText(mPasswordStr);
		
		//如果application正在登录， 则等待其完成
		while(((YuntuApplication)getApplication()).isLogging){
			myHandler.sendEmptyMessage(LOGGING);
		}

		// 如果已经登录直接跳转
		if (((YuntuApplication) getApplication()).isLogged) {
			myHandler.sendEmptyMessage(LOGINFINISH);
			doJumpMapTableList();
		} else {
			// application登录不成功，则再执行一次登录
			if (mNameStr != null && mPasswordStr != null) {
				requestLogin(mNameStr, mPasswordStr);
			}
		}
	}

	/**
	 * 改变登录按钮的状态
	 * 
	 * @param isLogging
	 *            是否正在登录
	 */
	private void changeLoginState(boolean isLogging) {
		((YuntuApplication) getApplication()).isLogging = isLogging;
		if (isLogging) {
			btLogin.setText(getResources().getString(R.string.logging));
			btLogin.setEnabled(false);
		} else {
			btLogin.setText(getResources().getString(R.string.login));
			btLogin.setEnabled(true);
		}
	}

	Handler myHandler = new Handler() {

		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case LOGGING:
				changeLoginState(true);
				break;
			case LOGINFINISH:
				changeLoginState(false);
				break;
			}
		}

	};

	/**
	 * 跳转到地图列表界面
	 */
	private void doJumpMapTableList() {
		Intent intent = new Intent(LoginActivity.this,
				MapTableListActivity.class);
		startActivity(intent);
		finish();
	}

	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.btLogin:
			mStrName = mUserName.getText().toString();
			mStrPassword = mPassword.getText().toString();
			if (mStrName.equals("")) {
				Util.showToast(getApplicationContext(), "请输入用户名");
				mUserName.setBackgroundResource(R.drawable.custom_background_edittext_login_err);
				mUserName.requestFocus();
				return;
			}
			mUserName.setBackgroundResource(R.drawable.custom_background_edittext_login);
			if (mStrPassword.equals("")) {
				Util.showToast(getApplicationContext(), "请输入密码");
				mPassword.setBackgroundResource(R.drawable.custom_background_edittext_login_err);
				mPassword.requestFocus();
				return;
			}
			mPassword.setBackgroundResource(R.drawable.custom_background_edittext_login);
			requestLogin(mStrName, mStrPassword);
			break;
		default:
			break;
		}
	}

	private synchronized void requestLogin(final String userName,
			final String password) {
		//处理登录逻辑
	}
}
