package com.amap.app.cloud.activity;

import android.app.Activity;
import android.content.Context;
import android.graphics.drawable.ColorDrawable;
import android.util.DisplayMetrics;
import android.view.GestureDetector;
import android.view.GestureDetector.OnGestureListener;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.Button;
import android.widget.PopupWindow;
import android.widget.TextView;

import com.amap.app.cloud.R;
import com.amap.app.cloud.constants.CloudConstants;
import com.amap.app.cloud.util.ConfigableConst;
import com.amap.app.cloud.util.Util;

public class LogoutPopupWindow extends PopupWindow implements OnGestureListener {

	private Button btnLogout;
	private View mMenuView;
	private TextView tvUserName, tvVersion;

	private GestureDetector gestureScanner;

	public LogoutPopupWindow(Activity activity, OnClickListener itemsOnClick) {
		super(activity);
		LayoutInflater inflater = (LayoutInflater) activity
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		mMenuView = inflater.inflate(R.layout.popupwindow_logout, null);
		btnLogout = (Button) mMenuView.findViewById(R.id.bt_logout);
		tvUserName = (TextView) mMenuView.findViewById(R.id.tv_userName);
		tvVersion = (TextView) mMenuView.findViewById(R.id.tv_version);

		DisplayMetrics dm = new DisplayMetrics();
		activity.getWindowManager().getDefaultDisplay().getMetrics(dm);

		gestureScanner = new GestureDetector(activity, this);

		String version = Util.getVersion(activity);
		tvUserName.setText(ConfigableConst.userName);
		tvVersion.setText(version);
		// 设置按钮监听
		btnLogout.setOnClickListener(itemsOnClick);
		// 设置PopupWindow的View
		this.setContentView(mMenuView);
		// 设置PopupWindow弹出窗体的宽
		this.setWidth(LayoutParams.MATCH_PARENT);
		// 设置PopupWindow弹出窗体的高
		this.setHeight(LayoutParams.MATCH_PARENT);
		// 设置SPopupWindow弹出窗体可点击
		this.setFocusable(true);
		// 设置PopupWindow弹出窗体动画效果
		this.setAnimationStyle(R.style.AnimLeft);
		// 实例化一个ColorDrawable颜色为半透明
		ColorDrawable dw = new ColorDrawable(0xb0000000);
		// 设置PopupWindow弹出窗体的背景
		this.setBackgroundDrawable(dw);
		// mMenuView添加OnTouchListener监听判断获取触屏位置如果在选择框外面则销毁弹出框
		mMenuView.setOnTouchListener(new OnTouchListener() {
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				gestureScanner.onTouchEvent(event);
				return true;
			}
		});
	}

	@Override
	public boolean onDown(MotionEvent e) {
		return false;
	}

	@Override
	public void onShowPress(MotionEvent e) {

	}

	@Override
	public boolean onSingleTapUp(MotionEvent e) {
		int width = mMenuView.findViewById(R.id.pop_layout).getRight();
		if (e.getX() > width) {
			dismiss();
		}
		return true;
	}

	@Override
	public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX,
			float distanceY) {
		return false;
	}

	@Override
	public void onLongPress(MotionEvent e) {

	}

	@Override
	public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX,
			float velocityY) {
		if ((e1.getX() - e2.getX()) > CloudConstants.SCROLL_MINPIX) {
			dismiss();
		} else {
			int width = mMenuView.findViewById(R.id.pop_layout).getRight();
			if (e1.getX() > width) {
				dismiss();
			}
		}
		return true;
	}
}
