/**
 */
package com.amap.app.cloud.activity;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationListener;
import com.amap.api.location.LocationManagerProxy;
import com.amap.api.location.LocationProviderProxy;
import com.amap.api.maps.AMap;
import com.amap.api.maps.AMap.OnCameraChangeListener;
import com.amap.api.maps.AMap.OnMapClickListener;
import com.amap.api.maps.CameraUpdateFactory;
import com.amap.api.maps.MapView;
import com.amap.api.maps.model.BitmapDescriptorFactory;
import com.amap.api.maps.model.CameraPosition;
import com.amap.api.maps.model.LatLng;
import com.amap.api.maps.model.Marker;
import com.amap.api.maps.model.MarkerOptions;
import com.amap.api.services.core.LatLonPoint;
import com.amap.api.services.geocoder.GeocodeResult;
import com.amap.api.services.geocoder.GeocodeSearch;
import com.amap.api.services.geocoder.GeocodeSearch.OnGeocodeSearchListener;
import com.amap.api.services.geocoder.RegeocodeQuery;
import com.amap.api.services.geocoder.RegeocodeResult;
import com.amap.app.cloud.R;
import com.amap.app.cloud.util.Util;

/**
 * @Description: TODO
 * @Project: YuntuConsoleAPP
 * @Class: MapLocationActivity
 * @date 2015年1月27日 上午10:41:45
 */
public class MapLocationActivity extends Activity implements
		OnCameraChangeListener, AMapLocationListener, OnMapClickListener,
		OnClickListener, OnGeocodeSearchListener {

	private ProgressDialog progDialog = null;
	private AMap amap;
	private MapView mapView;
	private LocationManagerProxy amapManager = null;
	private GeocodeSearch geocoderSearch;
	// 是否已经定位
	private boolean isLocationed = false;

	private LinearLayout llyBack;
	private LinearLayout llComplete;
	private TextView tvLocatinAddr;

	private String locationAddr;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.activity_maplocation);
		mapView = (MapView) findViewById(R.id.mapView);
		mapView.onCreate(savedInstanceState);
		amap = mapView.getMap();
		amap.setOnCameraChangeListener(this);
		
		geocoderSearch = new GeocodeSearch(this);
		geocoderSearch.setOnGeocodeSearchListener(this);

		llyBack = (LinearLayout) findViewById(R.id.title_lly_back);
		llComplete = (LinearLayout) findViewById(R.id.title_lly_add);
		tvLocatinAddr = (TextView) findViewById(R.id.tv_locationAddr);

		llyBack.setOnClickListener(this);
		llComplete.setOnClickListener(this);
		startLoaction();
	}

	/**
	 * 开始定位
	 */
	public void startLoaction() {
		showProgressDialog();
		amapManager = LocationManagerProxy.getInstance(this);
		if (amapManager != null) {
			amapManager.requestLocationData(LocationProviderProxy.AMapNetwork,
					-1, 10, this);
		}
	}

	/**
	 * 停止定位
	 */
	public void stopLocation() {
		if (amapManager != null) {
			amapManager.removeUpdates(this);
			amapManager.destroy();
			amapManager = null;
		}
	}

	/**
	 * 显示进度框
	 */
	private void showProgressDialog() {
		if (progDialog == null)
		progDialog = new ProgressDialog(this);
		progDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
		progDialog.setIndeterminate(false);
		progDialog.setCancelable(false);
		progDialog.setMessage("正在定位...");
		progDialog.show();
	}

	/**
	 * 隐藏进度框
	 */
	private void dissmissProgressDialog() {
		if (progDialog != null && progDialog.isShowing()) {
			progDialog.dismiss();
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		mapView.onResume();
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		mapView.onSaveInstanceState(outState);
	}

	@Override
	protected void onPause() {
		super.onPause();
		mapView.onPause();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		stopLocation();
		mapView.onPause();
	}

	@Override
	public void onCameraChange(CameraPosition cameraPosition) {

	}

	/**
	 * 响应逆地理编码
	 */
	private void getAddress(final LatLonPoint latLonPoint) {
		RegeocodeQuery query = new RegeocodeQuery(latLonPoint, 10,
				GeocodeSearch.AMAP);
		geocoderSearch.getFromLocationAsyn(query);// 设置同步逆地理编码请求
	}
	
	@Override
	public void onCameraChangeFinish(CameraPosition cameraPosition) {
		if(null != cameraPosition){
			LatLng latLng = cameraPosition.target; 
			LatLonPoint latLogPoint = new LatLonPoint(latLng.latitude, latLng.longitude);
			getAddress(latLogPoint);
		}
	}

	@Override
	public void onLocationChanged(Location location) {
		// TODO 自动生成的方法存根

	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO 自动生成的方法存根
	}
	

	@Override
	public void onProviderEnabled(String provider) {
		// TODO 自动生成的方法存根

	}

	@Override
	public void onProviderDisabled(String provider) {
		// TODO 自动生成的方法存根

	}

	@Override
	public void onLocationChanged(AMapLocation amapLocation) {
		dissmissProgressDialog();
		if (amapLocation != null
				&& amapLocation.getAMapException().getErrorCode() == 0) {
			amap.moveCamera(CameraUpdateFactory.newLatLngZoom(new LatLng(
					amapLocation.getLatitude(), amapLocation.getLongitude()),
					18));
			stopLocation();
			
			Marker amapMarker = amap.addMarker(new MarkerOptions()
					.icon(BitmapDescriptorFactory
							.defaultMarker(BitmapDescriptorFactory.HUE_BLUE)));
			amapMarker.setPosition(new LatLng(amapLocation.getLatitude(),
					amapLocation.getLongitude()));

			tvLocatinAddr.setText(amapLocation.getAddress());
			isLocationed = true;
		} else {
			tvLocatinAddr.setText("定位失败，请稍后重试");
			isLocationed = false;
		}
	}

	
	private void sendMsg(){
		Intent intent = getIntent();
		intent.putExtra("addr", tvLocatinAddr.getText());
		setResult(RESULT_OK, intent);
		
	}
	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.title_lly_add:
			sendMsg();
			MapLocationActivity.this.finish();
			break;
		case R.id.title_lly_back:
			sendMsg();
			finish();
			break;
		}
	}

	@Override
	public void onMapClick(LatLng arg0) {
		// 如果定位失败，重新进行一次定位
		if (!isLocationed) {
			startLoaction();
		}
	}

	@Override
	public void onGeocodeSearched(GeocodeResult arg0, int arg1) {
		
	}

	@Override
	public void onRegeocodeSearched(RegeocodeResult result, int rCode) {
		if (rCode == 0) {
			if (result != null && result.getRegeocodeAddress() != null
					&& result.getRegeocodeAddress().getFormatAddress() != null) {
				tvLocatinAddr.setText(result.getRegeocodeAddress().getFormatAddress());
			} else {
				Util.showToast(getApplicationContext(), getResources().getString(R.string.no_result));
			}
		} else if (rCode == 27) {
			Util.showToast(getApplicationContext(), getResources().getString(R.string.error_network));
		} else if (rCode == 32) {
			Util.showToast(getApplicationContext(), getResources().getString(R.string.error_key));
		} else {
			Util.showToast(getApplicationContext(), "位置搜索失败");
		}
	}

}
