package com.amap.app.cloud.activity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.LinearLayout;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;

import com.XListView.XListView;
import com.XListView.XListView.IXListViewListener;
import com.amap.app.cloud.R;
import com.amap.app.cloud.adapter.ListViewAdapter_MapTable;
import com.amap.app.cloud.dao.DataBaseAdapter;
import com.amap.app.cloud.net.Protocal;
import com.amap.app.cloud.pojo.CreateItemResultEntity;
import com.amap.app.cloud.pojo.DatabaseItemEntity;
import com.amap.app.cloud.pojo.TableResultEntity;
import com.amap.app.cloud.pojo.UploadPictureResultEntity;
import com.amap.app.cloud.util.ConfigableConst;
import com.amap.app.cloud.util.ServerError;
import com.amap.app.cloud.util.UploadRequestCallBack;
import com.amap.app.cloud.util.Util;
import com.lidroid.xutils.exception.HttpException;
import com.lidroid.xutils.http.ResponseInfo;
import com.lidroid.xutils.http.callback.RequestCallBack;

/**
 * @Description: 地图相信信息Activity
 * @Project: AMapCloud
 * @Class: MapTableDataActivity
 * @date 2015年1月14日 下午6:05:51
 */
public class MapTableDataActivity extends Activity implements
		OnItemClickListener, OnClickListener, OnCheckedChangeListener,
		IXListViewListener {

	private LinearLayout llyBack = null;
	private LinearLayout llyAdd = null;
	private LinearLayout llyUpload = null;
	private LinearLayout llyNoData = null;
	private XListView lvMapInfo;
	ListViewAdapter_MapTable adapter;
	private RadioGroup rgOption;
	private ProgressDialog dialog;

	// 是否只显示未上传数据
	public boolean isLocalDataList = false;

	// 本地数据
	private List<DatabaseItemEntity> localDataList = new ArrayList<DatabaseItemEntity>();

	// 已上传数据的Map
	private Map<Integer, List<DatabaseItemEntity>> serverDataMap = new HashMap<Integer, List<DatabaseItemEntity>>();

	// 所有数据
	public List<DatabaseItemEntity> allDataList = null;
	public static MapTableDataActivity instance;

	private Protocal mProtocal;
	private int pageNum = 1;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		mProtocal = ConfigableConst.getProtocalInstance(this
				.getApplicationContext());
		setContentView(R.layout.activity_maptabledata);
		lvMapInfo = (XListView) findViewById(R.id.lv_mapInfo);
		llyBack = (LinearLayout) findViewById(R.id.title_lly_back);
		llyAdd = (LinearLayout) findViewById(R.id.title_lly_add);
		llyUpload = (LinearLayout) findViewById(R.id.title_lly_upload);
		rgOption = (RadioGroup) findViewById(R.id.title_rg_option);
		llyNoData = (LinearLayout) findViewById(R.id.lly_noData);

		allDataList = new ArrayList<DatabaseItemEntity>();
		if (null == instance) {
			instance = this;
		}
		adapter = new ListViewAdapter_MapTable(this);
		adapter.setViewList(allDataList);
		lvMapInfo.setAdapter(adapter);

		lvMapInfo.setOnItemClickListener(this);
		lvMapInfo.setPullLoadEnable(true);
		lvMapInfo.setPullRefreshEnable(true);
		lvMapInfo.setXListViewListener(this);
		llyBack.setOnClickListener(this);
		llyAdd.setOnClickListener(this);
		llyUpload.setOnClickListener(this);
		rgOption.setOnCheckedChangeListener(this);
		dialog = new ProgressDialog(this);
		dialog.setCancelable(false);
		dialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		loadAllDataList(ConfigableConst.tableID);
		checkList();
	}

	@Override
	protected void onResume() {
		super.onResume();
		if (isLocalDataList) {
			rgOption.check(R.id.title_rb_waitUpload);
			lvMapInfo.setPullLoadEnable(false);
			lvMapInfo.setPullRefreshEnable(false);
			adapter.setViewList(localDataList);
		} else {
			rgOption.check(R.id.title_rb_all);
			adapter.setViewList(allDataList);
		}
		adapter.notifyDataSetChanged();
	}

	@Override
	protected void onDestroy() {
		if (null != instance) {
			instance = null;
		}
		super.onDestroy();
	}

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position,
			long id) {
		Intent intent = new Intent(this, ShowDataInfoActivity.class);
		// 由于使用XListView，增加了hearView，所以真实位置应该为position-1；
		intent.putExtra("index", (position - 1));
		startActivity(intent);
	}

	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.title_lly_back:
			finish();
			break;
		case R.id.title_lly_add:
			Intent intent = new Intent(this, AddDataInfoActivity.class);
			startActivity(intent);
			break;
		case R.id.title_lly_upload:
			checkBeforeUpload();
			break;
		}
	}

	/**
	 * 显示无网络连接对话框
	 */
	private void showNetworkDialog() {
		AlertDialog.Builder builder = new Builder(this);
		builder.setTitle("网络提示");
		builder.setMessage(getResources().getString(R.string.noNetwork));
		builder.setCancelable(false);
		builder.setPositiveButton(R.string.confirm, null);
		builder.create().show();
	}

	private void showWifiDialog() {
		AlertDialog.Builder alert = new AlertDialog.Builder(this);
		alert.setTitle("网络提示");
		alert.setMessage(getResources().getString(R.string.notInWifi));
		alert.setCancelable(false);
		alert.setPositiveButton("仍然上传", new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				upload();
			}
		});
		alert.setNegativeButton("取消", new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
			}
		});
		alert.show();
	}

	/**
	 * 上传前的检测
	 */
	private void checkBeforeUpload() {
		if (!Util.isNetworkConnected(this)) {
			showNetworkDialog();
			return;
		}
		upload();
	}

	private void upload() {
		if (localDataList == null || localDataList.size() == 0) {
			return;
		}
		final DatabaseItemEntity dataEntity = localDataList.get(0);
		dialog.setTitle(getResources().getString(R.string.uploading)
				+ dataEntity.getName());
		dialog.setMessage(getResources().getString(R.string.uploadingData));
		dialog.setIndeterminate(true);
		if (!dialog.isShowing()) {
			dialog.show();
		}
		dataCallBack.setEntity(dataEntity);
		mProtocal.createItem(ConfigableConst.tableID, ConfigableConst.sid,
				dataEntity, dataCallBack);
	}

	private boolean hasUpload() {
		if (localDataList == null || localDataList.size() == 0) {
			return false;
		} else {
			return true;
		}
	}

	UploadRequestCallBack dataCallBack = new UploadRequestCallBack() {

		@Override
		public void onSuccess(ResponseInfo<String> responseInfo) {
			CreateItemResultEntity resultEntity = mProtocal.parseResult(
					responseInfo, CreateItemResultEntity.class);
			if (resultEntity == null) {
				dialog.cancel();
				Util.showToast(getApplicationContext(), "上传失败");
				return;
			}
			DatabaseItemEntity dataEntity = getEntity();
			if (dataEntity == null) {
				dialog.cancel();
				return;
			}
			switch (resultEntity.getRc()) {
			case 1:
				ArrayList<byte[]> photoList = dataEntity.getPhotoList();
				if (photoList == null || photoList.size() == 0) {
					localDataList.remove(dataEntity);
					if (allDataList != null) {
						allDataList.remove(dataEntity);
					}
					deleteData(dataEntity.getId());
					adapter.notifyDataSetChanged();
					if (hasUpload()) {
						upload();
					} else {
						dialog.dismiss();
						dialog.cancel();
						rgOption.check(R.id.title_rb_all);
						onRefresh();
						refreshMapTableList();
					}
				} else {
					String dataID = resultEntity.getId();
					photoCallBack.setEntity(dataEntity);
					dialog.setMessage(getResources().getString(
							R.string.uploadingImg));
					dialog.setIndeterminate(false);
					dialog.setMax(100);
					mProtocal.uploadImage(ConfigableConst.tableID,
							ConfigableConst.sid, dataID, null, photoList,
							photoCallBack);
				}
				break;
			case 0:
				dialog.dismiss();
				Util.showToast(getApplicationContext(), "上传失败，未知原因");
				break;
			case -1:
				dialog.dismiss();
				Util.showToast(getApplicationContext(), "上传失败，tableId不存在");
				break;
			case -11:
				dialog.dismiss();
				Util.showToast(getApplicationContext(), "上传失败，数据类型非法");
				break;
			case -12:
				dialog.dismiss();
				Util.showToast(getApplicationContext(), "上传失败，数据长度超限");
				break;
			case -13:
				dialog.dismiss();
				Util.showToast(getApplicationContext(), "上传失败，包含敏感词");
				break;
			case -21:
				dialog.dismiss();
				Util.showToast(getApplicationContext(), "上传失败，用量达到上限");
				break;
			default:
				dialog.dismiss();
				break;

			}
		}

		@Override
		public void onFailure(HttpException error, String msg) {
			dialog.dismiss();
			Util.showToast(getApplicationContext(), ConfigableConst.NET_ERROR);
		}
	};

	UploadRequestCallBack photoCallBack = new UploadRequestCallBack() {

		@Override
		public void onLoading(long total, long current, boolean isUploading) {
			if (isUploading) {
				int progress = (int) (((current * 1.0f) / total) * 100);
				dialog.setProgress(progress);
			}
		}

		@Override
		public void onSuccess(ResponseInfo<String> responseInfo) {
			String[] resJSONArray = responseInfo.result.split("\\}");
			for (int i = 0; i < resJSONArray.length; i++) {
				resJSONArray[i] = resJSONArray[i] + "}";
			}
			int failureCount = 0;
			for (int i = 0; i < resJSONArray.length; i++) {
				UploadPictureResultEntity picEntity = mProtocal.parseResult(
						resJSONArray[i], UploadPictureResultEntity.class);
				if (picEntity == null) {
					continue;
				}
				DatabaseItemEntity dataEntity = getEntity();
				if (dataEntity == null) {
					dialog.cancel();
					return;
				}
				if (picEntity.getRc() != 1) {
					failureCount++;
				}
				switch (picEntity.getRc()) {
				case 1:

					break;
				case 0:
					dialog.dismiss();
					Util.showToast(getApplicationContext(), "上传失败，未知原因");
					break;
				case -1:
					dialog.dismiss();
					Util.showToast(getApplicationContext(), "上传失败，tableId不存在");
					break;
				case -11:
					dialog.dismiss();
					Util.showToast(getApplicationContext(), "上传失败，数据类型非法");
					break;
				case -12:
					dialog.dismiss();
					Util.showToast(getApplicationContext(), "上传失败，数据长度超限");
					break;
				case -13:
					dialog.dismiss();
					Util.showToast(getApplicationContext(), "上传失败，包含敏感词");
					break;
				case -21:
					dialog.dismiss();
					Util.showToast(getApplicationContext(), "上传失败，用量达到上限");
					break;
				default:
					dialog.dismiss();
					break;

				}
			}
			DatabaseItemEntity dataEntity = getEntity();
			if (dataEntity == null) {
				dialog.cancel();
				return;
			}
			localDataList.remove(dataEntity);
			if (allDataList != null) {
				allDataList.remove(dataEntity);
			}
			deleteData(dataEntity.getId());
			adapter.notifyDataSetChanged();
			if (failureCount > 0) {
				Util.showToast(getApplicationContext(), +failureCount
						+ "张照片上传失败");
			} else {
				if (hasUpload()) {
					upload();
				} else {
					dialog.dismiss();
					rgOption.check(R.id.title_rb_all);
					onRefresh();
					refreshMapTableList();
				}
			}
		}

		@Override
		public void onFailure(HttpException error, String msg) {
			dialog.dismiss();
			refreshMapTableList();
			// 上传图片失败，认为数据成功，所以删除数据
			DatabaseItemEntity dataEntity = getEntity();
			if (dataEntity == null) {
				return;
			}
			localDataList.remove(dataEntity);
			if (allDataList != null) {
				allDataList.remove(dataEntity);
			}
			deleteData(dataEntity.getId());
			adapter.notifyDataSetChanged();

			Util.showToast(getApplicationContext(), ConfigableConst.NET_ERROR);
		}
	};

	private void deleteData(long rowid) {
		DataBaseAdapter dataAdapter = new DataBaseAdapter(
				MapTableDataActivity.this);
		dataAdapter.open();
		dataAdapter.deleteTableData(rowid);
		dataAdapter.deletePhotoData(rowid);
		dataAdapter.close();
	}

	/**
	 * 刷新地图列表
	 */
	public void refreshMapTableList() {
		if (null != MapTableListActivity.instance) {
			MapTableListActivity.instance.onRefresh();
		}
	}

	@Override
	public void onCheckedChanged(RadioGroup group, int checkedId) {
		switch (checkedId) {
		case R.id.title_rb_all:
			isLocalDataList = false;
			lvMapInfo.setPullLoadEnable(true);
			lvMapInfo.setPullRefreshEnable(true);
			llyAdd.setVisibility(View.VISIBLE);
			llyUpload.setVisibility(View.GONE);
			isShowLoadMore();
			adapter.setViewList(allDataList);
			break;
		case R.id.title_rb_waitUpload:
			isLocalDataList = true;
			lvMapInfo.setPullLoadEnable(false);
			lvMapInfo.setPullRefreshEnable(false);
			llyAdd.setVisibility(View.GONE);
			llyUpload.setVisibility(View.VISIBLE);
			adapter.setViewList(localDataList);
			break;
		default:
			break;
		}
		adapter.notifyDataSetChanged();
	}

	@Override
	public void onRefresh() {
		pageNum = 1;
		loadAllDataList(ConfigableConst.tableID);
	}

	@Override
	public void onLoadMore() {
		// 加载更多的时候只加载服务端数据
		getServerDataList(ConfigableConst.tableID);
	}

	private void loadAllDataList(String tableId) {
		allDataList.removeAll(localDataList);
		localDataList = getLocalDataList(tableId);
		allDataList.addAll(0, localDataList);
		adapter.notifyDataSetChanged();
		getServerDataList(tableId);
	}

	/**
	 * 获取本地未上传数据
	 * 
	 * @return
	 */
	private List<DatabaseItemEntity> getLocalDataList(String tableid) {
		List<DatabaseItemEntity> dataList = new ArrayList<DatabaseItemEntity>();
		DataBaseAdapter dbAdapter = new DataBaseAdapter(this);
		dbAdapter.open();
		dataList = dbAdapter.fetchDataByTableID(tableid);
		dbAdapter.close();
		return dataList;
	}

	/**
	 * 从服务器获取已上传的数据
	 */
	private void getServerDataList(String tableId) {
		mProtocal.fetchTableContentList(tableId, ConfigableConst.sid, pageNum,
				requestMapTableListCallBack);
	}

	private RequestCallBack<String> requestMapTableListCallBack = new RequestCallBack<String>() {
		@Override
		public void onSuccess(ResponseInfo<String> responseInfo) {
			TableResultEntity tableResult = mProtocal
					.parseTableItemDataEntity(responseInfo);
			if (tableResult == null) {
				return;
			}
			int ec = tableResult.getEc();
			if (ec != 1) {
				Util.showToast(getApplicationContext(),
						ServerError.getMessage(ec));
				return;
			}

			int rc = tableResult.getRc();

			if (rc != 1) {
				switch (rc) {
				case 0:
					Util.showToast(getApplicationContext(), "获取数据失败，未知原因");
					break;
				case -1:
					Util.showToast(getApplicationContext(), getResources()
							.getString(R.string.mapMaybeDeleted));
					break;
				default:
					Util.showToast(getApplicationContext(), "获取数据失败");
					break;
				}
				lvMapInfo.stopRefresh();
				lvMapInfo.stopLoadMore();
				return;
			}
			List<DatabaseItemEntity> serverDataList = tableResult.getList();
			List<DatabaseItemEntity> oldServerList = serverDataMap.get(pageNum);
			serverDataMap.put(pageNum, serverDataList);

			if (null != oldServerList && oldServerList.size() > 0) {
				allDataList.removeAll(oldServerList);
			}
			allDataList.addAll(serverDataList);
			adapter.notifyDataSetChanged();

			if (serverDataList.size() >= ConfigableConst.TABLE_PAGESIZE) {
				pageNum++;
			}
			isShowLoadMore();
			lvMapInfo.stopRefresh();
			lvMapInfo.stopLoadMore();
			checkList();
		}

		@Override
		public void onFailure(HttpException error, String msg) {
			lvMapInfo.stopRefresh();
			lvMapInfo.stopLoadMore();
			checkList();
			Util.showToast(getApplicationContext(), ConfigableConst.NET_ERROR);
		}
	};

	/**
	 * 检查是否有数据
	 */
	private void checkList() {
		if (null == allDataList || allDataList.size() == 0) {
			lvMapInfo.setVisibility(View.GONE);
			llyNoData.setVisibility(View.VISIBLE);
		} else {
			llyNoData.setVisibility(View.GONE);
			lvMapInfo.setVisibility(View.VISIBLE);
		}
	}

	/**
	 * 判断是否显示加载更多
	 */
	private void isShowLoadMore() {
		if (serverDataMap == null) {
			return;
		}
		List<DatabaseItemEntity> serverDataList = serverDataMap.get(pageNum);
		if (serverDataList == null) {
			lvMapInfo.setPullLoadEnable(true);
			return;
		}
		if (pageNum == 1
				&& serverDataList.size() < ConfigableConst.TABLE_PAGESIZE) {
			lvMapInfo.setPullLoadEnable(false);
		} else {
			lvMapInfo.setPullLoadEnable(true);
		}
	}

}
