package com.amap.app.cloud.activity;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.AlertDialog.Builder;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.XListView.XListView;
import com.XListView.XListView.IXListViewListener;
import com.amap.app.cloud.R;
import com.amap.app.cloud.adapter.ListViewAdapter_MapTable;
import com.amap.app.cloud.net.Protocal;
import com.amap.app.cloud.pojo.AllMapResultEntity;
import com.amap.app.cloud.pojo.MapTableEntity;
import com.amap.app.cloud.util.ConfigableConst;
import com.amap.app.cloud.util.ServerError;
import com.amap.app.cloud.util.Util;
import com.lidroid.xutils.exception.HttpException;
import com.lidroid.xutils.http.ResponseInfo;
import com.lidroid.xutils.http.callback.RequestCallBack;

/**
 * @Project: AMapCloud
 * @Class: MyMapActivity
 * @Description: MapTableListActivity
 * @date 2015年1月14日 下午3:36:05
 */
public class MapTableListActivity extends Activity implements
		OnItemClickListener, OnClickListener, IXListViewListener {

	private static final int FINISHED = 0;
	private static final int LOADING = 1;

	public static MapTableListActivity instance = null;

	private XListView lvMapInfo;
	private ListViewAdapter_MapTable adapter;
	private Protocal mProtocal;
	private ProgressBar pbLoading;
	private TextView tvTableCount;
	private List mapTableList;

	private LinearLayout titleLlyUser;

	private LinearLayout titleLlyShare;

	private View parentView;
	// 自定义的弹出框类
	private LogoutPopupWindow menuWindow;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		mProtocal = ConfigableConst.getProtocalInstance(this
				.getApplicationContext());
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_maptablelist);
		pbLoading = (ProgressBar) findViewById(R.id.pb_loading);
		lvMapInfo = (XListView) findViewById(R.id.lv_mapInfo);
		titleLlyUser = (LinearLayout) findViewById(R.id.title_lly_user);
		tvTableCount = (TextView) findViewById(R.id.title_tv_mapNum);
		parentView = findViewById(R.id.myMap);
		titleLlyShare = (LinearLayout) findViewById(R.id.title_lly_share);
		if (null == instance) {
			instance = this;
		}

		adapter = new ListViewAdapter_MapTable(this);
		lvMapInfo.setAdapter(adapter);
		lvMapInfo.setOnItemClickListener(this);
		lvMapInfo.setPullLoadEnable(true);
		lvMapInfo.setPullRefreshEnable(true);
		lvMapInfo.setXListViewListener(this);
		titleLlyUser.setOnClickListener(this);
		titleLlyShare.setOnClickListener(this);
		lvMapInfo.setPullLoadEnable(true);
		titleLlyShare.setVisibility(View.GONE);
	}

	@Override
	protected void onResume() {
		super.onResume();
		myHandler.sendEmptyMessage(LOADING);
		getAllTable();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		instance = null;
	}

	Handler myHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case FINISHED:
				pbLoading.setVisibility(View.GONE);
				lvMapInfo.setVisibility(View.VISIBLE);
				adapter.notifyDataSetChanged();
				break;
			case LOADING:
				lvMapInfo.setVisibility(View.GONE);
				pbLoading.setVisibility(View.VISIBLE);
				break;
			}
		}
	};

	private void getAllTable() {
		mProtocal.fetchAllTable(ConfigableConst.sid,
				requestMapTableListCallBack);
	}

	private RequestCallBack<String> requestMapTableListCallBack = new RequestCallBack<String>() {

		@Override
		public void onSuccess(ResponseInfo<String> responseInfo) {
			AllMapResultEntity allMapResultEntity = mProtocal.parseResult(
					responseInfo, AllMapResultEntity.class);
			if (allMapResultEntity == null) {
				myHandler.sendEmptyMessage(FINISHED);
				return;
			}

			int ec = allMapResultEntity.getEc();
			if (ec != 1) {
				Util.showToast(getApplicationContext(),
						ServerError.getMessage(ec));
				return;
			}

			int rc = allMapResultEntity.getRc();
			if (rc != 1) {
				myHandler.sendEmptyMessage(FINISHED);
				return;
			}

			mapTableList = allMapResultEntity.getList();
			Collections.sort(mapTableList, new Comparator<MapTableEntity>() {
				@Override
				public int compare(MapTableEntity lhs, MapTableEntity rhs) {
					return Long.valueOf(rhs.getUpdateTime()).compareTo(
							Long.valueOf(lhs.getUpdateTime()));
				}
			});
			adapter.setViewList(mapTableList);
			StringBuffer sb = new StringBuffer();
			sb.append("(");
			sb.append(mapTableList.size());
			sb.append(")");
			tvTableCount.setText(sb.toString());
			lvMapInfo.stopRefresh();
			lvMapInfo.stopLoadMore();
			myHandler.sendEmptyMessage(FINISHED);
			// lvMapInfo.setAdapter(adapter);
		}

		@Override
		public void onFailure(HttpException error, String msg) {
			lvMapInfo.stopRefresh();
			lvMapInfo.stopLoadMore();
			Util.showToast(getApplicationContext(), ConfigableConst.NET_ERROR);
			myHandler.sendEmptyMessage(FINISHED);
		}
	};

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position,
			long id) {
		MapTableEntity itemView = (MapTableEntity) adapter
				.getItem((position - 1));
		Intent intent = new Intent(this, MapTableDataActivity.class);
		ConfigableConst.tableID = itemView.getId();
		startActivity(intent);
	}

	private void showMenu() {
		menuWindow = new LogoutPopupWindow(MapTableListActivity.this,
				menuOnClick);
		// 显示窗口
		menuWindow.showAtLocation(parentView, Gravity.LEFT
				| Gravity.CENTER_HORIZONTAL, 0, 0);
	}

	// 为图片选择弹出窗口实现监听类
	private OnClickListener menuOnClick = new OnClickListener() {
		public void onClick(View v) {
			menuWindow.dismiss();
			switch (v.getId()) {
			case R.id.bt_logout:
				showLogoutDialog(true);
				break;
			}
		}
	};

	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.title_lly_user:
			showMenu();
			break;
		case R.id.title_lly_share:
			break;
		default:
			break;
		}
	}

	private long exitTime = 0;

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK
				&& event.getAction() == KeyEvent.ACTION_DOWN) {
			if ((System.currentTimeMillis() - exitTime) > 2000) {
				Util.showToast(getApplicationContext(), "再按一次退出程序");
				exitTime = System.currentTimeMillis();
			} else {
				finish();
				System.exit(0);
			}
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}

	/**
	 * 清除数据
	 * 
	 * @param isShowLogin
	 *            是否显示登录界面
	 */
	private void doLogout(boolean isShowLogin) {
		ConfigableConst.getProtocalInstance(this.getApplicationContext())
				.exitLogin(null);
		ConfigableConst.sid = null;
		ConfigableConst.userName = null;
		((YuntuApplication) getApplication()).isLogged = false;
		if (isShowLogin) {
			Intent intent = new Intent(this, LoginActivity.class);
			startActivity(intent);
			finish();
		} else {
			finish();
			System.exit(0);
		}
	}

	private void showLogoutDialog(final boolean cleanLogin) {
		AlertDialog.Builder builder = new Builder(MapTableListActivity.this);
		builder.setMessage("确定退出?");
		builder.setCancelable(false);
		builder.setPositiveButton(R.string.confirm,
				new android.content.DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						dialog.dismiss();
						if (cleanLogin) {
							SharedPreferences sharedata = getSharedPreferences(
									"SP", MODE_PRIVATE);
							SharedPreferences.Editor editor = sharedata.edit();
							editor.remove("NAME");
							editor.remove("PASSWORD");
							editor.commit();
						}
						doLogout(cleanLogin);
					}
				});
		builder.setNegativeButton(R.string.cancel,
				new android.content.DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						dialog.dismiss();
					}
				});
		builder.create().show();
	}

	@Override
	public void onRefresh() {
		getAllTable();
	}

	@Override
	public void onLoadMore() {
		getAllTable();
	}

}
