/**
 */
package com.amap.app.cloud.activity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.app.Activity;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.amap.app.cloud.R;
import com.amap.app.cloud.adapter.Adapter_GridView_PhotoList;
import com.amap.app.cloud.callback.ImageCallback;
import com.amap.app.cloud.constants.CloudConstants;
import com.amap.app.cloud.pojo.ImageBucket;
import com.amap.app.cloud.pojo.ImageItem;
import com.amap.app.cloud.util.AlbumHelper;
import com.amap.app.cloud.util.BitmapCache;
import com.amap.app.cloud.util.ImageUtil;
import com.amap.app.cloud.util.Util;

/**
 * @Description: 照片列表Activity
 * @Project: YuntuConsoleAPP
 * @Class: PhotoListActivity
 * @date 2015年1月29日 下午3:30:51
 */
public class PhotoListActivity extends Activity implements OnClickListener {
	private GridView imgGridView;
	private AlbumHelper helper;
	private Adapter_GridView_PhotoList imgsAdapter;

	private Spinner spAlbum;

	private TextView tvAlbumName;
	private TextView tvCount;
	private LinearLayout llyComplete;
	private LinearLayout llyBack;

	private Map<String, Bitmap> selectedPhotoMap = null;

	ImageCallback callback = new ImageCallback() {
		@Override
		public void onImageLoaded(ImageView imageView, Bitmap bitmap,
				Object... params) {
			if (imageView != null && bitmap != null) {
				String url = (String) params[0];
				if (url != null && url.equals((String) imageView.getTag())) {
					((ImageView) imageView).setImageBitmap(bitmap);
				}
			}
		}
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		setContentView(R.layout.activity_photolist);

		helper = AlbumHelper.getHelper();
		helper.init(getApplicationContext());

		imgGridView = (GridView) findViewById(R.id.gv_photoList);
		llyBack = (LinearLayout) findViewById(R.id.title_lly_back);
		tvAlbumName = (TextView) findViewById(R.id.title_tv_page);
		tvCount = (TextView) findViewById(R.id.tv_count);
		llyComplete = (LinearLayout) findViewById(R.id.lly_complete);
		spAlbum = (Spinner) findViewById(R.id.sp_album);

		selectedPhotoMap = new HashMap<String, Bitmap>();
		selectedPhotoMap.putAll(AddDataInfoActivity.instance.selectedPhotoMap);
		showCount();

		llyComplete.setOnClickListener(this);
		llyBack.setOnClickListener(this);

		imgsAdapter = new Adapter_GridView_PhotoList(this);
		imgsAdapter.setSelectedMap(selectedPhotoMap);
		imgsAdapter.setCallBack(callback);
		imgGridView.setAdapter(imgsAdapter);

		imgsAdapter
				.setOnItemClickListener(new Adapter_GridView_PhotoList.OnItemClickListener() {

					@Override
					public void onItemClick(ToggleButton view, int position,
							boolean isChecked, RadioButton chooseBt,
							List<ImageItem> imageList) {
						if (selectedPhotoMap.size() == CloudConstants.MAXPICNUM
								&& isChecked) {
							view.setChecked(false);
							chooseBt.setChecked(false);
							Util.showToast(getApplicationContext(), "最多可以选择"
									+ CloudConstants.MAXPICNUM + "张 照片");
							return;
						}
						if (isChecked) {
							view.setChecked(true);
							chooseBt.setChecked(true);
							String imgPath = imageList.get(position).imagePath;

							Bitmap bitmap = null;
							bitmap = ImageUtil.revisionImageSize(imgPath,
									CloudConstants.MAXPHOTOPIXEL);
							if (bitmap != null) {
								selectedPhotoMap.put(imgPath, bitmap);
							}
						} else {
							selectedPhotoMap.remove(imageList.get(position).imagePath);
							view.setChecked(false);
							chooseBt.setChecked(false);
						}
						showCount();
					}
				});

	}

	@Override
	protected void onResume() {
		super.onResume();
		final List<ImageBucket> albumList = helper.getImagesBucketList(true);
		List<String> nameList = new ArrayList<String>();
		for (int i = 0; i < albumList.size(); i++) {
			nameList.add(albumList.get(i).bucketName);
		}
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item, nameList) {
			BitmapCache cache = new BitmapCache();

			@Override
			public View getDropDownView(int position, View convertView,
					ViewGroup parent) {
				if (convertView == null) {
					convertView = getLayoutInflater().inflate(
							R.layout.item_list_album, parent, false);
				}
				ImageView ivAlbumThumb = (ImageView) convertView
						.findViewById(R.id.iv_thumb);
				TextView tvCount = (TextView) convertView
						.findViewById(R.id.tv_count);
				TextView tvAlbumName = (TextView) convertView
						.findViewById(R.id.tv_name);
				ImageView ivChecked = (ImageView) convertView
						.findViewById(R.id.item_iv_to);

				ImageBucket item = albumList.get(position);
				tvCount.setText(String.valueOf(item.count));
				tvAlbumName.setText(item.bucketName);
				if (item.imageList != null && item.imageList.size() > 0) {
					String thumbPath = item.imageList.get(0).thumbnailPath;
					String sourcePath = item.imageList.get(0).imagePath;
					cache.displayBmp(ivAlbumThumb, thumbPath, sourcePath,
							callback);
				} else {
					ivAlbumThumb.setImageBitmap(null);
				}
				ivChecked
						.setVisibility(spAlbum.getSelectedItemPosition() == position ? View.VISIBLE
								: View.INVISIBLE);
				return convertView;
			}
		};
		spAlbum.setAdapter(adapter);
		spAlbum.setOnItemSelectedListener(new Spinner.OnItemSelectedListener() {
			public void onItemSelected(AdapterView<?> parent, View view,
					int position, long id) {
				ImageBucket bucket = albumList.get(position);
				if (null != bucket) {
					tvAlbumName.setText(bucket.bucketName);
					imgsAdapter.setList(bucket.imageList);
					imgsAdapter.notifyDataSetChanged();
				}
			}

			public void onNothingSelected(AdapterView<?> parent) {

			}
		});
	}

	private void showCount() {
		StringBuffer sb = new StringBuffer();
		sb.append(selectedPhotoMap.size());
		sb.append("/");
		sb.append(CloudConstants.MAXPICNUM);
		tvCount.setText(sb.toString());
	}

	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.lly_complete:
			finish();
			AddDataInfoActivity.instance.selectedPhotoMap.clear();
			AddDataInfoActivity.instance.selectedPhotoMap
					.putAll(selectedPhotoMap);
			AddDataInfoActivity.instance.updateAdapter();
			break;
		case R.id.title_lly_back:
			selectedPhotoMap = null;
			finish();
		}
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			selectedPhotoMap = null;
			finish();
		}
		return super.onKeyDown(keyCode, event);
	}

}
