/**
 */
package com.amap.app.cloud.adapter;

import java.util.List;
import java.util.Map;

import android.content.Context;
import android.graphics.Bitmap;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.RadioButton;
import android.widget.ToggleButton;

import com.amap.app.cloud.R;
import com.amap.app.cloud.callback.ImageCallback;
import com.amap.app.cloud.pojo.ImageItem;
import com.amap.app.cloud.util.BitmapCache;

/**
 * @Description: 显示已经选择的图片的适配器
 * @Project: YuntuConsoleAPP
 * @Class: GridViewAdapter_SelectedPic
 * @date 2015年1月19日 下午2:53:51
 */
public class Adapter_GridView_PhotoList extends BaseAdapter {

	private Context context;
	private List<ImageItem> dataList;
	private BitmapCache cache;

	private ImageCallback callback = null;
	private Map<String, Bitmap> selectedPhotoMap;

	/**
	 * 存放列表项控件句柄
	 */
	private class ViewHolder {
		public ImageView imageView;
		public ToggleButton toggleButton;
		public RadioButton choosetoggle;
	}

	public Adapter_GridView_PhotoList(Context context,
			List<ImageItem> dataList, Map<String, Bitmap> selectedPhotoMap) {
		this.context = context;
		this.dataList = dataList;
		setSelectedMap(selectedPhotoMap);
		cache = new BitmapCache();
	}

	public Adapter_GridView_PhotoList(Context context) {
		this.context = context;
		cache = new BitmapCache();
	}

	public void setSelectedMap(Map<String, Bitmap> selectedPhotoMap) {
		this.selectedPhotoMap = selectedPhotoMap;
	}

	public void setList(List<ImageItem> dataList) {
		this.dataList = dataList;
	}

	public void setCallBack(ImageCallback callback) {
		this.callback = callback;
	}

	@Override
	public int getCount() {
		if (null == dataList) {
			return 0;
		}
		return dataList.size();
	}

	@Override
	public Object getItem(int position) {
		if (null == dataList) {
			return null;
		}
		return dataList.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		ViewHolder viewHolder;
		if (convertView == null) {
			viewHolder = new ViewHolder();
			convertView = LayoutInflater.from(context).inflate(
					R.layout.item_photolist, parent, false);
			viewHolder.imageView = (ImageView) convertView
					.findViewById(R.id.image_view);
			viewHolder.toggleButton = (ToggleButton) convertView
					.findViewById(R.id.toggle_button);
			viewHolder.choosetoggle = (RadioButton) convertView
					.findViewById(R.id.choosedbt);
			convertView.setTag(viewHolder);
		} else {
			viewHolder = (ViewHolder) convertView.getTag();
		}
		final ImageItem item = dataList.get(position);

		viewHolder.imageView.setTag(item.imagePath);
		cache.displayBmp(viewHolder.imageView, item.thumbnailPath,
				item.imagePath, callback);

		viewHolder.toggleButton.setTag(position);
		viewHolder.choosetoggle.setTag(position);
		viewHolder.toggleButton.setOnClickListener(new ToggleClickListener(
				viewHolder.choosetoggle));
		if (selectedPhotoMap.containsKey(dataList.get(position).imagePath)) {
			viewHolder.toggleButton.setChecked(true);
			viewHolder.choosetoggle.setChecked(true);
		} else {
			viewHolder.toggleButton.setChecked(false);
			viewHolder.choosetoggle.setChecked(false);
		}
		return convertView;
	}

	private class ToggleClickListener implements OnClickListener {
		RadioButton chooseBt;

		public ToggleClickListener(RadioButton choosebt) {
			this.chooseBt = choosebt;
		}

		@Override
		public void onClick(View view) {
			if (view instanceof ToggleButton) {
				ToggleButton toggleButton = (ToggleButton) view;
				int position = (Integer) toggleButton.getTag();
				if (dataList != null && mOnItemClickListener != null
						&& position < dataList.size()) {
					mOnItemClickListener.onItemClick(toggleButton, position,
							toggleButton.isChecked(), chooseBt, dataList);
				}
			}
		}
	}

	private OnItemClickListener mOnItemClickListener;

	public void setOnItemClickListener(OnItemClickListener listener) {
		mOnItemClickListener = listener;
	}

	public interface OnItemClickListener {
		public void onItemClick(ToggleButton view, int position,
				boolean isChecked, RadioButton chooseBt,
				List<ImageItem> dataList);
	}
}
