/**
 */
package com.amap.app.cloud.adapter;

import java.util.List;

import android.app.Activity;
import android.graphics.Bitmap;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.amap.app.cloud.R;
import com.amap.app.cloud.callback.ImageCallback;
import com.amap.app.cloud.pojo.ImageBucket;
import com.amap.app.cloud.util.BitmapCache;

/**
 * @Description: 相册的适配器
 * @Project: YuntuConsoleAPP
 * @Class: Adapter_ListView_Album
 * @date 2015年1月29日 下午1:49:19
 */
public class Adapter_ListView_Album extends BaseAdapter {
	private Activity activity;
	private List<ImageBucket> dataList;
	private BitmapCache cache;
	
	ImageCallback callback = new ImageCallback() {
		@Override
		public void onImageLoaded(ImageView imageView, Bitmap bitmap,
				Object... params) {
			if (imageView != null && bitmap != null) {
				String url = (String) params[0];
				if (url != null && url.equals((String) imageView.getTag())) {
					((ImageView) imageView).setImageBitmap(bitmap);
				}
			}
		}
	};

	public Adapter_ListView_Album(Activity activity, List<ImageBucket> dataList) {
		this.activity = activity;
		this.dataList = dataList;
		cache = new BitmapCache();
	}

	class Holder {
		// 相册的缩略图
		public ImageView ivAlbumThumb;
		// 相册中包含的照片数量
		public TextView tvCount;
		// 相册名称
		public TextView tvAlbumName;
	}

	@Override
	public int getCount() {
		if (null == dataList) {
			return 0;
		}
		return dataList.size();
	}

	@Override
	public Object getItem(int position) {
		if (null == dataList) {
			return null;
		}
		return dataList.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(final int position, View convertView, ViewGroup parent) {
		Holder holder;
		if(null == convertView){
			holder = new Holder();
			convertView = View.inflate(activity,
					R.layout.item_list_album, null);
			holder.ivAlbumThumb = (ImageView) convertView
					.findViewById(R.id.iv_thumb);
			holder.tvCount = (TextView) convertView.findViewById(R.id.tv_count);
			holder.tvAlbumName = (TextView) convertView
					.findViewById(R.id.tv_name);
			convertView.setTag(holder);
		} else {
			holder = (Holder) convertView.getTag();
		}
		
		ImageBucket item = dataList.get(position);
		holder.tvCount.setText(String.valueOf(item.count));
		holder.tvAlbumName.setText(item.bucketName);
		if (item.imageList != null && item.imageList.size() > 0) {
			String thumbPath = item.imageList.get(0).thumbnailPath;
			String sourcePath = item.imageList.get(0).imagePath;
			holder.ivAlbumThumb.setTag(sourcePath);
			cache.displayBmp(holder.ivAlbumThumb, thumbPath, sourcePath, callback);
		} else {
			holder.ivAlbumThumb.setImageBitmap(null);
		}
		return convertView;
	}
}
