/**
 */
package com.amap.app.cloud.adapter;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.ImageView;

import com.amap.app.cloud.R;
import com.amap.app.cloud.constants.CloudConstants;
import com.amap.app.cloud.util.Util;

/**
 * @Description: 显示已经选择的图片的适配器
 * @Project: YuntuConsoleAPP
 * @Class: GridViewAdapter_SelectedPic
 * @date 2015年1月19日 下午2:53:51
 */
public class GridViewAdapter_SelectedPic extends BaseAdapter {
	private LayoutInflater inflater;
	private Context context;
	private List<DataItem> dataList;
	
	private class DataItem{
		String imgPath;
		Bitmap bitmap;
	}

	// 视图控件集合
	private class ViewHolder {
		public ImageView image;
		public Button btDel;
	}
	
	public GridViewAdapter_SelectedPic(Context context, Map<String, Bitmap> itemMap) {
		this.context = context;
		inflater = LayoutInflater.from(context);
		this.dataList = getListFromMap(itemMap);
	}
	
	public void setItemMap(Map<String, Bitmap> itemMap){
		this.dataList = getListFromMap(itemMap);
	}
	
	private List<DataItem> getListFromMap(Map<String, Bitmap> map){
		List<DataItem> itemList = new ArrayList<DataItem>();
		Iterator<Entry<String, Bitmap>> iterator = map.entrySet().iterator();
		while(iterator.hasNext()){
			Entry<String, Bitmap> entry = iterator.next();
			
			DataItem item = new DataItem();
			byte[] b = Util.getBitmapByte(entry.getValue());
			item.bitmap = entry.getValue();
			item.imgPath = entry.getKey();
			itemList.add(item);
		}
		return itemList;
	}

	public int getCount() {
		if (dataList.size() < CloudConstants.MAXPICNUM) {
			return dataList.size() + 1;
		} else {
			return CloudConstants.MAXPICNUM;
		}

	}

	public Object getItem(int positon) {
		return null;
	}

	public long getItemId(int arg0) {
		return arg0;
	}

	public View getView(int position, View convertView, ViewGroup parent) {
		ViewHolder holder = null;
		if (convertView == null) {
			convertView = inflater.inflate(R.layout.item_grid_selectedpic,
					parent, false);
			holder = new ViewHolder();
			holder.image = (ImageView) convertView
					.findViewById(R.id.item_grida_image);
			holder.btDel = (Button) convertView.findViewById(R.id.bt_delSelect);
			convertView.setTag(holder);
		} else {
			holder = (ViewHolder) convertView.getTag();
		}

		holder.image.setImageBitmap(BitmapFactory.decodeResource(
				context.getResources(), R.drawable.camera));
		if (dataList.size() == 0) {
			holder.image.setImageBitmap(BitmapFactory.decodeResource(
					context.getResources(), R.drawable.camera));
			holder.btDel.setVisibility(View.GONE);
		} else {
			holder.image.setTag(position);
			holder.btDel.setTag(position);
			if (position == dataList.size()) {
				holder.image.setImageBitmap(BitmapFactory.decodeResource(
						context.getResources(), R.drawable.camera));
				holder.btDel.setVisibility(View.GONE);
			} else {
				holder.image.setImageBitmap(dataList.get(position).bitmap);
				holder.image.setOnClickListener(new ItemClickListener());
				holder.btDel.setOnClickListener(new ItemClickListener());
			}
		}
		return convertView;
	}

	private class ItemClickListener implements OnClickListener {
		@Override
		public void onClick(View view) {
			if (mOnItemClickListener != null && dataList.size() > 0) {
				if (view instanceof ImageView) {
					ImageView iv = (ImageView) view;
					int position = (Integer) iv.getTag();
					String imgPath = dataList.get(position).imgPath;
					mOnItemClickListener.onImgClick(position, imgPath);
				}
				if (view instanceof Button) {
					Button bt = (Button) view;
					int position = (Integer) bt.getTag();
					String imgPath = dataList.get(position).imgPath;
					mOnItemClickListener.onDelBtClick(position, imgPath);
				}
			}

		}
	}

	private OnViewClickListener mOnItemClickListener;

	public void setOnViewClickListener(OnViewClickListener listener) {
		mOnItemClickListener = listener;
	}

	public interface OnViewClickListener {
		/**
		 * 点击图片时的监听
		 * @param position
		 */
		public void onImgClick(int position, String imgPath);
		/**
		 * 点击删除图片时的监听
		 * @param position
		 */
		public void onDelBtClick(int position, String imgPath);
	}
}
