package com.amap.app.cloud.adapter;

import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.amap.app.cloud.R;
import com.amap.app.cloud.pojo.MapTableEntity;
import com.amap.app.cloud.pojo.TableItemDataEntity;
import com.amap.app.cloud.util.Util;

/**
 * 我的地图列表适配器
 * 
 * @Description: TODO
 * @Project: YuntuConsoleAPP
 * @Class: ListViewAdapter_MapTable
 * @date 2015年1月23日 下午2:10:09
 */
public class ListViewAdapter_MapTable extends BaseAdapter {
	private List viewList;
	private LayoutInflater layoutInflater;

	private static String dateFormat = "yyyy/MM/dd HH:mm:ss";

	public ListViewAdapter_MapTable(Context context) {
		this.layoutInflater = LayoutInflater.from(context);
	}

	public void setViewList(List viewList) {
		this.viewList = viewList;
	}

	@Override
	public int getCount() {
		if (null == viewList) {
			return 0;
		}
		return viewList.size();
	}

	@Override
	public Object getItem(int position) {
		if (null == viewList) {
			return null;
		}
		return viewList.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {

		if (null == viewList) {
			return null;
		}
		ListViewItem itemView = null;
		if (convertView == null) {
			itemView = new ListViewItem();
			// 获得组件，实例化组件
			convertView = layoutInflater.inflate(R.layout.item_list_maptable,
					null);
			itemView.tvTitle = (TextView) convertView
					.findViewById(R.id.tv_name);
			itemView.tvInfo = (TextView) convertView
					.findViewById(R.id.item_tv_info);
			itemView.tvLastUpdate = (TextView) convertView
					.findViewById(R.id.item_tv_lastUpdate);
			itemView.ivAlertUpload = (ImageView) convertView
					.findViewById(R.id.item_alert_upload);
			itemView.ivAlertUpload.setVisibility(View.GONE);
			convertView.setTag(itemView);
		} else {
			itemView = (ListViewItem) convertView.getTag();
		}

		Object viewInfo = viewList.get(position);

		if (viewInfo instanceof MapTableEntity) {
			MapTableEntity mapTableEntity = (MapTableEntity) viewInfo;
			itemView.tvTitle.setText(mapTableEntity.getName());
			StringBuffer infoStr = new StringBuffer();
			infoStr.append("个数:");
			infoStr.append(mapTableEntity.getCount());
			infoStr.append(" | ");
			infoStr.append("大小:");
			infoStr.append(mapTableEntity.getStorage() + "KB");
			itemView.tvInfo.setText(infoStr.toString());
			itemView.tvLastUpdate.setVisibility(View.VISIBLE);
			itemView.tvLastUpdate.setText("更新于 " + Util.formatDate(
					mapTableEntity.getUpdateTime(), dateFormat));
		}

		if (viewInfo instanceof TableItemDataEntity) {
			TableItemDataEntity itemDataEntity = (TableItemDataEntity) viewInfo;
			itemView.tvTitle.setText(itemDataEntity.getName());
			itemView.tvInfo.setText(itemDataEntity.getAddress());
			itemView.tvLastUpdate.setVisibility(View.GONE);
			if (itemDataEntity.getCreateTime() == 0L) {
				itemView.ivAlertUpload.setVisibility(View.VISIBLE);
			} else {
				itemView.ivAlertUpload.setVisibility(View.GONE);
			}
		}

		return convertView;
	}
}
