package com.amap.app.cloud.dao;

import java.util.ArrayList;

import org.json.JSONException;
import org.json.JSONObject;

import com.amap.app.cloud.pojo.CreateItemRequestEntity;
import com.amap.app.cloud.pojo.DatabaseItemEntity;
import com.amap.app.cloud.pojo.GeoEntity;
import com.google.gson.Gson;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

public class DataBaseAdapter {
	// 用于打印log
	private static final String TAG = "DataBaseAdapter";
	// sqlite 自增序列必须名称必须以 _id 标示
	public static final String KEY_ID = "_id";
	// TABLE
	public static final String TABLE_NAME = "table_name";
	public static final String TABLE_ID = "table_id";
	public static final String TABLE_POST_TYPE = "table_post_type";
	public static final String TABLE_ADDRESS = "table_address";
	public static final String TABLE_GEO = "table_geo";
	public static final String TABLE_COORD_TYPE = "table_coored_type";
	public static final String TABLE_PROPS = "table_props";

	// photo
	public static final String PHOTO_ID = "photo_id";
	public static final String PHOTO_DATA = "photo_data";

	// 数据库名称
	private static final String DB_NAME = "yuntuUpdate.db";

	// 数据库表名
	private static final String DB_TABLE = "tableitem";
	private static final String DB_PHOTO = "photoitem";

	// 数据库版本
	private static final int DB_VERSION = 1;

	// 本地Context对象
	private Context mContext = null;

	// 创建一个表
	private static final String DB_CREATE_TABLE = "CREATE TABLE " + DB_TABLE
			+ "(" + KEY_ID + " INTEGER PRIMARY KEY," + TABLE_NAME + " TEXT,"
			+ TABLE_POST_TYPE + " INTEGER," + TABLE_ADDRESS + " TEXT,"
			+ TABLE_GEO + " TEXT," + TABLE_COORD_TYPE + " INTEGER,"
			+ TABLE_PROPS + " TABLE_PROPS," + TABLE_ID + " TEXT)";
	private static final String DB_CREATE_PHOTO = "CREATE TABLE " + DB_PHOTO
			+ "(" + KEY_ID + " INTEGER PRIMARY KEY," + PHOTO_DATA + " BLOB,"
			+ PHOTO_ID + " INTEGER)";

	// 执行open（）打开数据库时，保存返回的数据库对象
	private SQLiteDatabase mSQLiteDatabase = null;

	// 由SQLiteOpenHelper继承过来
	private DatabaseHelper mDatabaseHelper = null;
	private Gson gson;

	private static class DatabaseHelper extends SQLiteOpenHelper {

		/* 构造函数-创建一个数据库 */
		DatabaseHelper(Context context) {

			// 当调用getWritableDatabase()
			// 或 getReadableDatabase()方法时
			// 则创建一个数据库
			super(context, DB_NAME, null, DB_VERSION);
		}

		/* 创建一个表 */
		@Override
		public void onCreate(SQLiteDatabase db) {
			// 数据库没有表时创建一个
			db.execSQL(DB_CREATE_TABLE);
			db.execSQL(DB_CREATE_PHOTO);
		}

		/* 升级数据库 */
		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {

//			db.execSQL("DROP TABLE IF EXISTS notes");
//			onCreate(db);
		}
	}

	/* 构造函数-取得Context */
	public DataBaseAdapter(Context context) {
		mContext = context;
		gson = new Gson();
	}

	// 打开数据库，返回数据库对象
	public void open() throws SQLException {
		mDatabaseHelper = new DatabaseHelper(mContext);
		mSQLiteDatabase = mDatabaseHelper.getWritableDatabase();
	}

	// 关闭数据库
	public void close() {
		mDatabaseHelper.close();
	}

	/* 插入一条数据 */
	public long insertTableData(DatabaseItemEntity entity) {
		String geo = gson.toJson(entity.getGeo());
		ContentValues initialValues = new ContentValues();
		initialValues.put(TABLE_NAME, entity.getName());
		initialValues.put(TABLE_POST_TYPE, entity.getPosType());
		initialValues.put(TABLE_ADDRESS, entity.getAddress());
		initialValues.put(TABLE_GEO, geo);
		initialValues.put(TABLE_COORD_TYPE, entity.getCoordType());
		initialValues.put(TABLE_PROPS, entity.getProps().toString());
		initialValues.put(TABLE_ID, entity.getTableID());
		return mSQLiteDatabase.insert(DB_TABLE, TABLE_NAME, initialValues);
	}

	public long insertPhotoData(long tablerowID, byte[] photodata) {
		ContentValues initialValues = new ContentValues();
		initialValues.put(PHOTO_DATA, photodata);
		initialValues.put(PHOTO_ID, tablerowID);
		return mSQLiteDatabase.insert(DB_PHOTO, null, initialValues);
	}

	public long insertData(DatabaseItemEntity entity,
			ArrayList<byte[]> photoList) {
		long rowID = insertTableData(entity);
		if (photoList == null || photoList.size() == 0) {
			return rowID;
		}
		for (byte[] photoData : photoList) {
			insertPhotoData(rowID, photoData);
		}
		return rowID;
	}

	/* 删除一条数据 */
	public boolean deletePhotoData(long rowID) {
		Cursor cursor = fetchPhotoData(rowID);
		if(cursor.getCount() == 0){
			return true;
		}
		return mSQLiteDatabase.delete(DB_PHOTO, PHOTO_ID + "=" + rowID, null) > 0;
	}

	public boolean deleteTableData(long rowID) {
		return mSQLiteDatabase.delete(DB_TABLE, KEY_ID + "=" + rowID, null) > 0;
	}

	// public boolean deleteData(long rowID) {
	// boolean tableResult = deleteTableData(rowID);
	// boolean photoResult = deletePhotoData(rowID);
	// return tableResult && photoResult;
	// }

	/* 通过Cursor查询所有数据 */
	public ArrayList<DatabaseItemEntity> fetchDataByTableID(String tableid) {
		Cursor cursor = mSQLiteDatabase.query(DB_TABLE, new String[] { KEY_ID,
				TABLE_NAME, TABLE_POST_TYPE, TABLE_ADDRESS, TABLE_GEO,
				TABLE_COORD_TYPE, TABLE_PROPS, TABLE_ID }, TABLE_ID + "=" + "'"
				+ tableid + "'", null, null, null, KEY_ID + " desc");
		if (cursor == null) {
			return null;
		}
		ArrayList<DatabaseItemEntity> list = cursor2ItemEntityList(cursor);
		for (DatabaseItemEntity entity : list) {
			long tablerowid = entity.getId();
			Cursor photoCursor = fetchPhotoData(tablerowid);
			if (photoCursor == null) {
				continue;
			}
			ArrayList<byte[]> photoList = cursor2PhotoList(photoCursor);
			entity.setPhotoList(photoList);
		}
		return list;
	}

	/* 查询指定数据 */

	public DatabaseItemEntity fetchData(long tablerowID) throws SQLException {
		Cursor cursor = fetchTableData(tablerowID);
		if (cursor == null) {
			return null;
		}
		cursor.moveToFirst();
		DatabaseItemEntity entity = cursor2ItemEntity(cursor);
		Cursor photoCursor = fetchPhotoData(tablerowID);
		if (photoCursor == null) {
			return entity;
		}
		ArrayList<byte[]> photoList = cursor2PhotoList(photoCursor);
		if (entity != null) {
			entity.setPhotoList(photoList);
		}
		return entity;
	}

	private Cursor fetchPhotoData(long tableRowid) {
		String sql = "SELECT  *  FROM  photoitem where photoitem.photo_id =?";
		Cursor photoCursor = mSQLiteDatabase.rawQuery(sql, new String[] { ""
				+ tableRowid });
		return photoCursor;
	}

	private Cursor fetchTableData(long rowId) throws SQLException {
		Cursor mCursor = mSQLiteDatabase.query(true, DB_TABLE, new String[] {
				KEY_ID, TABLE_NAME, TABLE_POST_TYPE, TABLE_ADDRESS, TABLE_GEO,
				TABLE_COORD_TYPE, TABLE_PROPS }, KEY_ID + "=" + rowId, null,
				null, null, null, null);
		if (mCursor != null) {
			mCursor.moveToFirst();
		}
		return mCursor;
	}

	private ArrayList<DatabaseItemEntity> cursor2ItemEntityList(Cursor cursor) {
		ArrayList<DatabaseItemEntity> list = new ArrayList<DatabaseItemEntity>();
		while (cursor.moveToNext()) {
			DatabaseItemEntity entity = cursor2ItemEntity(cursor);
			list.add(entity);
		}
		return list;
	}

	private DatabaseItemEntity cursor2ItemEntity(Cursor cursor) {
		DatabaseItemEntity entity = new DatabaseItemEntity();
		entity.setId(getLong(cursor, KEY_ID));
		entity.setName(getString(cursor, TABLE_NAME));
		entity.setPosType(getInt(cursor, TABLE_POST_TYPE));
		entity.setAddress(getString(cursor, TABLE_ADDRESS));
		String geoString = getString(cursor, TABLE_GEO);
		if (geoString != null) {
			GeoEntity geoentity = gson.fromJson(geoString, GeoEntity.class);
			entity.setGeo(geoentity);
		}
		entity.setCoordType(getInt(cursor, TABLE_COORD_TYPE));
		String propString = getString(cursor, TABLE_PROPS);
		if (propString != null) {
			JSONObject json;
			try {
				json = new JSONObject(propString);
				entity.setProps(json);
			} catch (JSONException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return entity;
	}

	private ArrayList<byte[]> cursor2PhotoList(Cursor cursor) {
		ArrayList<byte[]> photoList = new ArrayList<byte[]>();
		while (cursor.moveToNext()) {
			byte[] photo = getBytes(cursor, PHOTO_DATA);
			photoList.add(photo);
		}
		return photoList;
	}

	private int getInt(Cursor cursor, String columnName) {
		int index = cursor.getColumnIndex(columnName);
		int keyValue = cursor.getInt(index);
		return keyValue;
	}

	private String getString(Cursor cursor, String columnName) {
		int index = cursor.getColumnIndex(columnName);
		String keyValue = cursor.getString(index);
		return keyValue;
	}

	private long getLong(Cursor cursor, String columnName) {
		int index = cursor.getColumnIndex(columnName);
		long keyValue = cursor.getLong(index);
		return keyValue;
	}

	private byte[] getBytes(Cursor cursor, String columnName) {
		int index = cursor.getColumnIndex(columnName);
		byte[] keyValue = cursor.getBlob(index);
		return keyValue;
	}

	/* 更新一条数据 */
	public boolean updateTableData(long rowId, CreateItemRequestEntity entity) {
		Gson gson = new Gson();
		String geo = gson.toJson(entity.getGeo());
		ContentValues initialValues = new ContentValues();
		initialValues.put(TABLE_NAME, entity.getName());
		initialValues.put(TABLE_POST_TYPE, entity.getPosType());
		initialValues.put(TABLE_ADDRESS, entity.getAddress());
		initialValues.put(TABLE_GEO, geo);
		initialValues.put(TABLE_COORD_TYPE, entity.getCoordType());
		initialValues.put(TABLE_PROPS, entity.getProps().toString());
		return mSQLiteDatabase.update(DB_TABLE, initialValues, KEY_ID + "="
				+ rowId, null) > 0;
	}

	public boolean updatePhotoData(long rowId, ArrayList<byte[]> photoList) {
		boolean result = deletePhotoData(rowId);
		if (null != photoList && photoList.size() > 0) {
			for (byte[] photoData : photoList) {
				insertPhotoData(rowId, photoData);
			}
		}
		return result;
	}
}