package com.amap.app.cloud.net;

import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.graphics.Bitmap;
import android.widget.ImageView;

import com.amap.app.cloud.pojo.CreateItemRequestEntity;
import com.amap.app.cloud.pojo.DatabaseItemEntity;
import com.amap.app.cloud.pojo.TableItemDataEntity;
import com.amap.app.cloud.pojo.TableResultEntity;
import com.google.gson.Gson;
import com.lidroid.xutils.BitmapUtils;
import com.lidroid.xutils.HttpUtils;
import com.lidroid.xutils.bitmap.callback.BitmapLoadCallBack;
import com.lidroid.xutils.http.ResponseInfo;
import com.lidroid.xutils.http.callback.RequestCallBack;

public class Protocal {
	private HttpUtils http;
	private Gson gson;
	private BitmapUtils getBitmap;
	private ImageView iView;
	private static final String multipart_form_data = "multipart/form-data";
	private Context mContext;

	public Protocal(Context context, ImageView view) {
		mContext = context;
		getBitmap = new BitmapUtils(context);
		if (view == null) {
			iView = new ImageView(context);
		} else {
			iView = view;
		}
	}

	public Protocal(Context context) {
		mContext = context;
		http = new HttpUtils();
		http.configHttpCacheSize(0);
		http.configDefaultHttpCacheExpiry(0);
		gson = new Gson();
		getBitmap = new BitmapUtils(context);
	}

	/**
	 * 获取所有table
	 * 
	 * @param sid
	 * @param callBack
	 */
	public void fetchAllTable(String sid, RequestCallBack<String> callBack) {
		//从云图数据管理台获取tableid数据
		//http://lbs.amap.com/yuntu/
	}

	/**
	 * 登录 获取koken
	 * 
	 * @param username
	 * @param password
	 * @param callBack
	 */
	public void fetchLogin(String username, String password,
			RequestCallBack<String> callBack) {
		// 通过用户账号体系获取token
	}

	/**
	 * 退出登录
	 * 
	 * @param callBack
	 */
	public void exitLogin(RequestCallBack<String> callBack) {
		// 退出登录
	}

	/**
	 * 新增一条数据
	 * 
	 * @param item
	 * @param callBack
	 */
	public void createItem(String tableid, String sid,
			CreateItemRequestEntity item, RequestCallBack<String> callBack) {
		// 创建一条数据,参考云图新增数据接口,地址如下
		// http://lbs.amap.com/yuntu/reference/cloudstorage/
	}

	private String item2String(CreateItemRequestEntity item) {
		if (item == null) {
			return null;
		}
		JSONObject obj = null;
		try {
			CreateItemRequestEntity entity = new CreateItemRequestEntity();
			entity.setAddress(item.getAddress());
			entity.setCoordType(item.getCoordType());
			entity.setGeo(item.getGeo());
			entity.setName(item.getName());
			entity.setPosType(item.getPosType());
			entity.setProps(entity.getProps());
			String jsonString = gson.toJson(entity);
			obj = new JSONObject(jsonString);
			obj.remove("props");
			obj.put("props", item.getProps());
		} catch (JSONException e) {
			e.printStackTrace();
		}
		return obj.toString();
	}

	public void fetchTableContentList(String tableid, String sid,
			int pageNumber, RequestCallBack<String> callBack) {
		// 获取已上传云图数据，参考下方地址文档描述
		// http://lbs.amap.com/yuntu/reference/cloudstorage/
	}

	public void fetchTableStruct(String tableid, String sid,
			RequestCallBack<String> callBack) {
		// 获取已云图自定义字段结构
		// http://lbs.amap.com/yuntu/reference/cloudstorage/
	}

	public void downloadImageView(String url,
			BitmapLoadCallBack<ImageView> callBack) {
		getBitmap.configMemoryCacheEnabled(false);
		getBitmap.configDiskCacheEnabled(false);
		getBitmap.display(iView, url, callBack);
	}

	public void downloadImageView(String url, ImageView imageView,
			BitmapLoadCallBack<ImageView> callBack) {
		getBitmap.configMemoryCacheEnabled(true);
		getBitmap.configDiskCacheEnabled(true);
		getBitmap.display(imageView, url, callBack);
	}

	/**
	 * 上传图片接口 可以传递bitmap list或者 byte[] list
	 * 
	 * @param tableId
	 * @param sid
	 * @param dataId
	 * @param bitmapList
	 *            bitmap list
	 * @param byteList
	 *            byte[] list
	 * @param callBack
	 *            回调
	 */
	public void uploadImage(String tableId, String sid, String dataId,
			ArrayList<Bitmap> bitmapList, ArrayList<byte[]> byteList,
			RequestCallBack<String> callBack) {
		// 上传图片到云图一条数据
		// http://lbs.amap.com/yuntu/reference/cloudstorage/

	}


	/**
	 * 解析json
	 * 
	 * @param <T>
	 * @param <T>
	 * @param <T>
	 * 
	 * @param response
	 * @return
	 */
	public <T> T parseResult(ResponseInfo<String> response, Class<T> classOfT) {
		if (response == null) {
			return null;
		}
		T entity = gson.fromJson(response.result, classOfT);
		return entity;
	}

	public <T> T parseResult(String jsonString, Class<T> classOfT) {
		if (jsonString == null) {
			return null;
		}
		T entity = gson.fromJson(jsonString, classOfT);
		return entity;
	}

	public TableResultEntity parseTableItemDataEntity(
			ResponseInfo<String> response) {
		TableResultEntity entity = null;
		if (response == null) {
			return null;
		}
		try {
			entity = gson.fromJson(response.result, TableResultEntity.class);
			if (entity == null) {
				return entity;
			}
			JSONObject jsonobj = new JSONObject(response.result);
			JSONArray jsonList = jsonobj.optJSONArray("list");
			if (jsonList == null) {
				return entity;
			}
			for (int i = 0; i < jsonList.length(); i++) {
				JSONObject itemJSON = jsonList.optJSONObject(i);
				if (itemJSON == null) {
					continue;
				}
				JSONObject propsJSON = itemJSON.optJSONObject("props");
				ArrayList<DatabaseItemEntity> list = entity.getList();
				if (list == null || list.size() == 0) {
					break;
				}
				TableItemDataEntity itemEntity = entity.getList().get(i);
				if (itemEntity != null) {
					itemEntity.setProps(propsJSON);
				}
			}
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return entity;
	}
}
