/**
 */
package com.amap.app.cloud.util;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.media.ExifInterface;
import android.util.DisplayMetrics;

/**
 * @Description: 图片工具类
 * @Project: YuntuConsoleAPP
 * @Class: ImageUtil
 * @date 2015年1月22日 下午5:18:49
 */
public class ImageUtil {
	public static final int PHOTO_QUALITY = 90;

	/**
	 * 按照旋转角度旋转bitmap
	 * 
	 * @param rotate
	 * @param bitmap
	 * @return
	 */
	public static Bitmap rotaingImageView(int rotate, Bitmap bitmap) {
		if (bitmap == null)
			return null;

		int w = bitmap.getWidth();
		int h = bitmap.getHeight();
		// Setting post rotate to 90
		Matrix mtx = new Matrix();
		mtx.postRotate(rotate);
		return Bitmap.createBitmap(bitmap, 0, 0, w, h, mtx, true);
	}

	/**
	 * 获取图片的旋转角度
	 * 
	 * @param path
	 * @return
	 */
	public static int readPictureDegree(String path) {
		int degree = 0;
		try {
			ExifInterface exifInterface = new ExifInterface(path);
			int orientation = exifInterface.getAttributeInt(
					ExifInterface.TAG_ORIENTATION,
					ExifInterface.ORIENTATION_NORMAL);
			switch (orientation) {
			case ExifInterface.ORIENTATION_ROTATE_90:
				degree = 90;
				break;
			case ExifInterface.ORIENTATION_ROTATE_180:
				degree = 180;
				break;
			case ExifInterface.ORIENTATION_ROTATE_270:
				degree = 270;
				break;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return degree;
	}

	/**
	 * @param filePath
	 *            存储路径
	 * @param dm
	 *            DisplayMetrics
	 * @return 长宽最大为720的位图
	 *
	 *         description 目前拍照功能在使用
	 */
	public static Bitmap image_Compression(String filePath, DisplayMetrics dm,
			int maxQuality) {
		BitmapFactory.Options opts = new BitmapFactory.Options();
		opts.inJustDecodeBounds = true;
		BitmapFactory.decodeFile(filePath, opts);

		/* update by yuchao.zhang 更新上传图片的分辨率压缩策略，最大长宽为720 */
		// int intScreenX=dm.widthPixels;
		// int intScreenY=dm.heightPixels;
		// int intWidth=intScreenX-40;
		// int intHeight=intScreenY-80;
		//
		// int bmHeight=opts.outHeight;
		// int bmWidth=opts.outWidth;
		//
		// int scaleW=bmWidth/intWidth;
		// int scaleH=bmHeight/intHeight;
		// int scale=scaleW>scaleH?scaleW:scaleH;
		// if(scale<=0)
		// scale=1;
		// opts.inSampleSize = scale;
		int widthRatio = (int) Math.ceil(opts.outWidth / maxQuality);
		int heightRatio = (int) Math.ceil(opts.outHeight / maxQuality);
		if (widthRatio > 1 || heightRatio > 1) {
			if (widthRatio > heightRatio) {
				opts.inSampleSize = widthRatio;
			} else {
				opts.inSampleSize = heightRatio;
			}
		}

		opts.inJustDecodeBounds = false;
		// opts.inPreferredConfig = Bitmap.Config.RGB_565;
		Bitmap bmp = null;
		try {
			bmp = BitmapFactory.decodeFile(filePath, opts);
			bmp = scaleImg(bmp, maxQuality);
		} catch (OutOfMemoryError e) {
			e.printStackTrace();
			try {
				bmp = BitmapFactory.decodeFile(filePath, opts);
				bmp = scaleImg(bmp, maxQuality);
			} catch (OutOfMemoryError ee) {
			}
		}

		return bmp;
	}

	/**
	 * 压缩图片
	 * 
	 * @param filePath
	 *            文件路径
	 * @param maxPixel
	 *            高度和宽度的最大像素值
	 * @return
	 */
	public static Bitmap image_Compression(String filePath, int maxPixel) {
		BitmapFactory.Options opts = new BitmapFactory.Options();
		opts.inJustDecodeBounds = true;
		BitmapFactory.decodeFile(filePath, opts);

		int widthRatio = (int) Math.ceil(opts.outWidth / maxPixel);
		int heightRatio = (int) Math.ceil(opts.outHeight / maxPixel);
		if (widthRatio > 1 || heightRatio > 1) {
			if (widthRatio > heightRatio) {
				opts.inSampleSize = widthRatio;
			} else {
				opts.inSampleSize = heightRatio;
			}
		}

		opts.inJustDecodeBounds = false;
		Bitmap bmp = null;
		try {
			bmp = BitmapFactory.decodeFile(filePath, opts);
			bmp = scaleImg(bmp, maxPixel);
		} catch (OutOfMemoryError e) {
			e.printStackTrace();
			try {
				bmp = BitmapFactory.decodeFile(filePath, opts);
				bmp = scaleImg(bmp, maxPixel);
			} catch (OutOfMemoryError ee) {
			}
		}
		return bmp;
	}

	public static void compressBmpToFile(Bitmap bmp, String filePath) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			bmp.compress(Bitmap.CompressFormat.JPEG, ImageUtil.PHOTO_QUALITY,
					baos);
			FileOutputStream fos = new FileOutputStream(filePath);
			fos.write(baos.toByteArray());
			fos.flush();
			fos.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static Bitmap scaleImg(Bitmap img, float quality) {

		if (img == null) {
			return null;
		}
		float sHeight = 0;
		float sWidth = 0;

		Bitmap result = img;

		if (img.getWidth() >= img.getHeight() && img.getWidth() > quality) {

			sWidth = quality;
			sHeight = (quality * img.getHeight()) / img.getWidth();
			result = Bitmap.createScaledBitmap(img, (int) sWidth,
					(int) sHeight, true);

			if (null != img && !img.isRecycled() && !img.equals(result)) {
				img.recycle();
			}
		} else if (img.getHeight() > img.getWidth()
				&& img.getHeight() > quality) {

			sHeight = quality;
			sWidth = (quality * img.getWidth()) / img.getHeight();
			result = Bitmap.createScaledBitmap(img, (int) sWidth,
					(int) sHeight, true);

			if (null != img && !img.isRecycled() && !img.equals(result)) {
				img.recycle();
			}
		}

		return result;
	}

	/**
	 * 根据图片路径旋转图片, 不压缩, 一般用于缩略图的旋转
	 * 
	 * @param path
	 * @return
	 */
	public static Bitmap rotaingImageView(String path) {
		// 获取旋转角度
		int degree = ImageUtil.readPictureDegree(path);
		Bitmap bitmap = BitmapFactory.decodeFile(path);
		return ImageUtil.rotaingImageView(degree, bitmap);
	}

	/**
	 * 按照指定的最大值压缩图片的大小
	 * 
	 * @param path
	 *            文件路径
	 * @param maxPixel
	 *            高度和宽度的最大像素值
	 * @return 压缩后的Bitmap
	 */
	public static Bitmap revisionImageSize(String path, int maxPixel) {
		// 获取旋转角度
		int degree = ImageUtil.readPictureDegree(path);
		// 压缩图片
		Bitmap bitmap = ImageUtil.image_Compression(path, maxPixel);
		// 旋转图片
		Bitmap newbitmap = ImageUtil.rotaingImageView(degree, bitmap);
		return newbitmap;
	}
}
